
/*
 *  Mac #includes.c - source to MacHeaders
 *
 *  To add a header, change its "#if 0" to "#if 1".
 *
 *  To remove a header, change its "#if 1" to "#if 0".
 *
 *  There are some conflicts and order dependencies among the various
 *  headers:
 *
 *		<LoMem.h> and <SysEqu.h> cannot both be included.
 *
 *		<asm.h> and <Traps.h>, if both are included, must appear
 *		in that order.  If <Traps.h> is included, traps used in
 *		inline assembly must appear without leading underscores.
 *
 *  If the "Check Pointer Types" option is disabled during the
 *  precompilation process, trap definitions will be stored in
 *  such a way that when a trap is called (in a source file that
 *  #includes the precompiled header), pointer arguments to the
 *  trap will not be matched against the types of corresponding
 *  formal parameters.
 *
 *  This is accomplished by storing "simplified" prototypes for
 *  traps, in which any argument of pointer type is stored as
 *  "void *".  The result closely resembles the treatment of traps
 *  in pre-5.0 versions of THINK C.
 *
 *  (Note that this file is written in such a way that it is immune
 *  to the actual compiler setting of "Check Pointer Types".  Use
 *  the "SIMPLIFY_PROTOTYPES" macro, below, to control whether full
 *  prototypes are retained.)
 *
 *	Note that none of the foregoing applies to Symantec C++, nor to 
 *	Symantec C/C++ for MPW. This file is #included by the source for
 *	MacHeaders++, the precompiled header for Symantec C++. If a compiler
 *	other than THINK C precompiles this file, asm.h and LoMem.h are
 *	#if'd out. If an MPW-hosted compiler precompiles this file, BDC.h
 *	and pascal.h are #if'd out.
 *
 */

#ifndef SystemSixOrLater
#define SystemSixOrLater 1
#endif

#ifndef SystemSevenOrLater
#define SystemSevenOrLater 1
#endif

#ifndef __SC__

// set this to 0 (zero) to retain full prototypes
// set this to 1 (one) for "simplified" prototypes
#define SIMPLIFY_PROTOTYPES		1


// prototype checking level
#if SIMPLIFY_PROTOTYPES
	#if !__option(check_ptrs)
		#undef SIMPLIFY_PROTOTYPES
	#endif
	#pragma options(!check_ptrs)
#else
	#if __option(check_ptrs)
		#undef SIMPLIFY_PROTOTYPES
	#endif
	#pragma options(check_ptrs)
#endif

#endif


//	#include <ADSP.h>
//	#include <AIFF.h>
//	#include <Aliases.h>		// Note: unnecessarily includes <AppleTalk.h>
//	#include <AppleEvents.h>	// Note: unnecessarily includes <EPPC.h>
//	#include <AppleTalk.h>
//	#include <Balloons.h>		// Note: unnecessarily includes <Traps.h> for _Pack14
	#include <BDC.h>
//	#include <CommResources.h>
//	#include <Connections.h>
//	#include <ConnectionTools.h>
	#include <Controls.h>
//	#include <CRMSerialDevices.h>
//	#include <CTBUtilities.h>
//	#include <DatabaseAccess.h>
	#include <Desk.h>
//	#include <DeskBus.h>
	#include <Devices.h>
	#include <Dialogs.h>
	#include <DiskInit.h>
//	#include <Disks.h>
//	#include <Editions.h>
//	#include <ENET.h>
//	#include <EPPC.h>
	#include <Errors.h>
	#include <Events.h>
	#include <Files.h>
//	#include <FileTransfers.h>
//	#include <FileTransferTools.h>
//	#include <Finder.h>
//	#include <FixMath.h>
	#include <Folders.h>
	#include <Fonts.h>
	#include <GestaltEqu.h>
//	#include <Graf3D.h>
//	#include <HyperXCmd.h>
//	#include <Icons.h>
//	#include <Language.h>
	#include <Lists.h>
	#include <Memory.h>
	#include <Menus.h>
//	#include <MIDI.h>
	#include <Notification.h>
	#include <OSEvents.h>
	#include <OSUtils.h>
//	#include <Packages.h>
//	#include <Palette.h>
//	#include <Palettes.h>
//	#include <Picker.h>
//	#include <PictUtil.h>
//	#include <Power.h>
//	#include <PPCToolBox.h>
//	#include <Printing.h>
	#include <PrintTraps.h>
//	#include <Processes.h>
//	#include <QDOffscreen.h>
	#include <Quickdraw.h>
	#include <Resources.h>
//	#include <Retrace.h>
//	#include <ROMDefs.h>
//	#include <SANE.h>
	#include <Scrap.h>
//	#include <Script.h>
//	#include <SCSI.h>
	#include <SegLoad.h>
//	#include <Serial.h>
//	#include <ShutDown.h>
//	#include <Slots.h>
//	#include <Sound.h>
//	#include <SoundInput.h>
	#include <StandardFile.h>
//	#include <Start.h>
//	#include <SysEqu.h>
//	#include <Terminals.h>
//	#include <TerminalTools.h>
	#include <TextEdit.h>
	#include <Timer.h>
	#include <ToolUtils.h>
//	#include <Traps.h>
	#include <Types.h>
//	#include <Values.h>
//	#include <Video.h>
	#include <Windows.h>
	#include <pascal.h>

	#ifdef THINK_C
		#include <asm.h>
	#endif

	#if !defined(__cplusplus)
		#include <LoMem.h>
	#endif

	#include <THINK.h>

#ifndef __SC__

// restore "Check Pointer Types" to previous setting
#if SIMPLIFY_PROTOTYPES
	#pragma options(check_ptrs)
#elif defined(SIMPLIFY_PROTOTYPES)
	#pragma options(!check_ptrs)
#endif
#undef SIMPLIFY_PROTOTYPES

#endif
